(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2009                                               *)
(*    CEA (Commissariat  l'nergie Atomique)                             *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* $Id: boot.ml,v 1.33 2008-11-18 12:13:41 uid568 Exp $ *)

(** Frama-C Entry Point (last linked module).
    @plugin development guide *)

(* ************************************************************************* *)
(** Registering options for debugging project *)
(* ************************************************************************* *)

let dump_dependencies () =
  let _except =
    Project.Selection.remove Parameters.Machdep.self
      (Project.Selection.remove Parameters.MainFunction.self
	 (Project.Selection.remove Parameters.LibEntry.self
	    (Plugin.get_selection ())))
  in
  Project.Computation.dump_dependencies (*~except*)
    "computation_dependencies.dot"

(* ************************************************************************* *)
(** Adding some internal state dependencies *)
(* ************************************************************************* *)

let () =
  Project.Computation.add_dependency Alarms.self Db.Value.self;
  List.iter
    Ast.depend
    [ Parameters.SimplifyCfg.self;
      Parameters.KeepSwitch.self;
      Parameters.UnrollingLevel.self;
      Parameters.Constfold.self;
      Parameters.ReadAnnot.self;
      Parameters.PreprocessAnnot.self ]

(* ************************************************************************* *)
(** Booting Frama-C *)
(* ************************************************************************* *)

(* Customisation of non-projectified CIL parameters.
   (projectified CIL parameters must be initialised with {!Cil.initCIL}). *)
let boot_cil () =
  Cabs2cil.forceRLArgEval := false;
  Cil.miscState.Cil.lineDirectiveStyle <- None;
  (*  Cil.lineDirectiveStyle := Some LinePreprocessorInput;*)
  Cil.miscState.Cil.printCilAsIs <- Parameters.Debug.get () > 0;
  Mergecil.ignore_merge_conflicts := true;
  Pretty.flushOften := true

(* Main: let's go! *)
let () =
  if Parameters.Debug.get () > 0 then Printexc.record_backtrace true;
  Kind.version := Config.version;
  boot_cil ();
  Sys.catch_break true;
  try
    Journal.start ();
    if not (Parameters.Journal.Enable.get ()) then begin
      Journal.clear ~restart:false ();
      Journal.stop ()
    end;
    if Parameters.Journal.Name.get () <> "" then
      Journal.set_name (Parameters.Journal.Name.get ());
    ignore (Project.create "default");
    Cmdline.parse_then_run_mains ();
    exit 0 (* ensure that nothing occurs after booting: no other file can be
	      linked after boot.ml *)
  with
  | Sys.Break as e -> (* User interuption *)
      raise e
  | Log.AbortError s ->
      Parameters.error
	"%s failed to complete because of an invalid user input" 
	s;
      exit 1
  | Log.AbortFatal s when Parameters.Debug.get () = 0 ->
      (* do it only in non-debugging mode: 
	 get a backtrace if you are debugging. *)
      Parameters.error
	"%s failed to complete because of an unexpected internal error.
please report at http://bts.frama-c.com"
	s;
      exit 2
  | e when Parameters.Debug.get () = 0 ->
      (* do it only in non-debugging mode: 
	 get a backtrace if you are debugging. *)
      Parameters.error 
	"unexpected error %s
please report as `crash' at http://bts.frama-c.com"
	(Printexc.to_string e);
      exit 3

(*
Local Variables:
compile-command: "LC_ALL=C make -C ../.. -j"
End:
*)
