(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2009                                               *)
(*    CEA (Commissariat  l'nergie Atomique)                             *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(* $id: cvalue.ml,v 1.36 2006/03/17 13:54:56 uid527 Exp $ *)
open Abstract_interp

(*
(** Initialize the default bound for absolute address to max addressable bit *)
let () = Parameters.max_valid_absolute_address := Utils.memory_size ()
*)

let main () =
  let not_quiet = not (Parameters.Quiet.get ()) in
  (* Memoization of context free functions *)
  let mem_functions = Value_parameters.MemFunctions.get () in
  if Value_parameters.MemExecAll.get ()
    || not (Cilutil.StringSet.is_empty mem_functions)
  then begin
    if not_quiet then Format.printf "====== MEMOIZING FUNCTIONS ======@.";
    Globals.Functions.iter
      (fun kf ->
	 let name = Kernel_function.get_name kf in
	 if Kernel_function.is_definition kf &&
	   (Value_parameters.MemExecAll.get () 
	    || Cilutil.StringSet.mem name mem_functions)
	 then begin
	   if not_quiet then 
	     Format.printf "== function %a@." 
	       Kernel_function.pretty_name kf;
	   try
	     !Db.Value.memoize kf
           with Db.Value.Aborted ->
	     Format.printf "Cannot memoize %a: Analysis degenerated@."
	       Kernel_function.pretty_name kf;
	     exit 1
	 end)
  end;
  (* Value computations *)
  if Value_parameters.ForceValues.get () then begin
    !Db.Value.compute ();
    if not_quiet then 
      begin
        Format.printf "@\n====== VALUES COMPUTED ======@.";
      end
  end;
  let forcederef = Parameters.Dynamic.Bool.get "-deref" in
  let forceout = Parameters.Dynamic.Bool.get "-out" in
  let forceinout = Parameters.Dynamic.Bool.get "-inout" in
  let forceinput = Parameters.Dynamic.Bool.get "-input" in
  let forceexternalout = Parameters.Dynamic.Bool.get "-out-external" in
  let forceinputwithformals = Parameters.Dynamic.Bool.get "-input-with-formals"
  in
  (* Computing others results and displaying results *)
  if forceout
    || forceexternalout
    || forceinput
    || forceinputwithformals
    || forceinout
    || forcederef
    || Value_parameters.ForceValues.get ()
  then begin
    !Db.Semantic_Callgraph.topologically_iter_on_functions
      (fun kf ->
	 if Kernel_function.is_definition kf then begin
	   let result display compute =
	     (if not_quiet then display Format.std_formatter else compute) kf
	   in
	   if forceout then 
	     result !Db.Outputs.display !Db.Outputs.compute;
	   if forceexternalout then 
	     result !Db.Outputs.display_external !Db.Outputs.compute;
	   if forceinput then
	     result !Db.Inputs.display !Db.Inputs.compute;
	   if forceinputwithformals then
	     if not_quiet then 
	       !Db.Inputs.display_with_formals Format.std_formatter kf
             (*else  !Db.Inputs.compute_with_formals) kf *) ;
	   if forceinout then
	     result !Db.InOutContext.display !Db.InOutContext.compute;
	   if forcederef then
	     result !Db.Derefs.display !Db.Derefs.compute;
	   if not_quiet && Value_parameters.ForceValues.get () then
	     Db.Value.display Format.std_formatter kf;
	 end)
  end

let () = Db.Main.extend main

(*
Local Variables:
compile-command: "LC_ALL=C make -C ../.. -j"
End:
*)
